
// NEOPIXEL BEST PRACTICES for most reliable operation:
// - Add 1000 uF CAPACITOR between NeoPixel strip's + and - connections.
// - MINIMIZE WIRING LENGTH between microcontroller board and first pixel.
// - NeoPixel strip's DATA-IN should pass through a 300-500 OHM RESISTOR.
// - AVOID connecting NeoPixels on a LIVE CIRCUIT. If you must, ALWAYS
//   connect GROUND (-) first, then +, then data.
// - When using a 3.3V microcontroller with a 5V-powered NeoPixel strip,
//   a LOGIC-LEVEL CONVERTER on the data line is STRONGLY RECOMMENDED.
// (Skipping these may work OK on your workbench but can fail in the field)


// Declare our NeoPixel strip object:
// Argument 1 = Number of pixels in NeoPixel strip
// Argument 2 = Arduino pin number (most are valid)
// Argument 3 = Pixel type flags, add together as needed:
//   NEO_KHZ800  800 KHz bitstream (most NeoPixel products w/WS2812 LEDs)
//   NEO_KHZ400  400 KHz (classic 'v1' (not v2) FLORA pixels, WS2811 drivers)
//   NEO_GRB     Pixels are wired for GRB bitstream (most NeoPixel products)
//   NEO_RGB     Pixels are wired for RGB bitstream (v1 FLORA pixels, not v2)
//   NEO_RGBW    Pixels are wired for RGBW bitstream (NeoPixel RGBW products)


/*
void showMap4x4()
// 10242/683 mit einer For und Berechnung
{
  const byte houses[] {34, 162, 42, 170};
  for (auto & i : houses)
  {
    for (byte j = 0; j < 4; j++)
    {
      strip.setPixelColor(i + j, houseColor);
      strip.setPixelColor((i % 16) + 16 * (i / 16 + 1) + j , houseColor);
      strip.setPixelColor(i + j + 32 , houseColor);
      strip.setPixelColor((i % 16) + 16 * (i / 16 + 3) + j , houseColor);
    }
  }
  //strip.show(); // with the first car
}
*/

/*
void gardenerTick()
{
  const byte action = RUIN;                // This company upgrades ruins to parks
  static uint32_t previousMillis = -300;
  static uint32_t previousMillisOrder = 0;
  static byte target = 0;
  static byte order = 0;
  static byte state = 0;                   // 0 ready // 1 Drive // 2 Arrival // 2 doing
  if (millis() - previousMillis > 500)     // yes, this action is slow - the car will block the street during the working time!
  {
    if (state == 0)                        // find a new target
    {
      for (byte i = 0; i < noOfHouses; i++)
      {
        if (city.state[i] == action && random(0, 3) == 1)
        {
          target = pgm_read_byte(&houses[i][1]); // die Straße
          order = i;                             // die Haus-ID
          //Serial.print(F("auftrag für ")); Serial.println(order);
          break;
        }
      }
      if (target > 0)
      {
        carGardener.setTarget(target, order);
        state = 1;
        previousMillisOrder = millis();
      }
      else
      {
        carGardener.setTarget(255, 0);
      }
    }
    else if (state == 1)                                    // drive to target
    {
      if (carGardener.currentPosition == target)
      {
        // Serial.println(F("arrived @")); Serial.println(pgm_read_byte(&houses[order][0]));
        state = 2;
      }
      if (millis() - previousMillisOrder > 50000) state = 0; // check if order isn't to old ...
      if (city.state[order] != action) state = 0;            // check, if house has changed its state, then cancel order
    }
    else if (state == 2)
    {
      if (city.state[order] == action)                       // if house has still the right status, do your action
      {
        city.state[order] = (action + 1) % 4;
        city.showHouse(order);
        //Serial.print(F("upgraded @")); Serial.println(pgm_read_byte(&houses[order][0]));
      }
      state = 0;
    }
    previousMillis = millis();
  }
}


void brickieTick()
{
  const byte action = PARK;                // This company upgrades parks to houses
  static uint32_t previousMillis = -1000;
  static uint32_t previousMillisOrder = 0;
  static byte target = 0;
  static byte order = 0;
  static byte state = 0;                   // 0 ready // 1 Drive // 2 Arrival // 2 doing
  if (millis() - previousMillis > 500)     // yes, this action is slow - the car will block the street during the working time!
  {
    // suche ein Ziel
    if (state == 0)
    {
      for (byte i = 0; i < noOfHouses; i++)
      {
        if (city.state[i] == action && random(0, 3) == 1)
        {
          target = pgm_read_byte(&houses[i][1]); // die Straße
          order = i;                             // die Haus-ID
          //Serial.print(F("auftrag für ")); Serial.println(order);
          break;
        }
      }
      if (target > 0)
      {
        carBrickie.setTarget(target, order);
        state = 1;
        previousMillisOrder = millis();
      }
      else
      {
        carBrickie.setTarget(255, 0);
      }
    }
    // Fahre mit dem Auto dort hin
    else if (state == 1)
    {
      // MISSING checken auf Zeitablauf ...
      if (carBrickie.currentPosition == target)
      {
        // Serial.println(F("arrived @")); Serial.println(pgm_read_byte(&houses[order][0]));
        state = 2;
      }
      if (millis() - previousMillisOrder > 50000) state = 0;
    }
    else if (state == 2)
    {
      // mach noch einen Check ober der Status noch passt
      if (city.state[order] == action)
      {
        // Wenn ja Mach deine Aktion mit der Parzelle
        city.state[order] = (action + 1) % 4;
        city.showHouse(order);
        //Serial.print(F("upgraded @")); Serial.println(pgm_read_byte(&houses[order][0]));
      }
      state = 0;
    }
    previousMillis = millis();
  }
}
*/



/*
// this will print a simple map of the city, showing 4 blocks with lot of "houses"
void showMapOld()
// 10188flash/695ram original
{
  const uint32_t houseColor = strip.Color(64,  64,  64);
  const byte houses[] {34, 35, 36, 37, 50, 51, 52, 53, 66, 67, 68, 69, 82, 83, 84, 85};
  for (auto & i : houses)
  {
    strip.setPixelColor(i, houseColor);
    strip.setPixelColor(i - 34 + 42, houseColor);
    strip.setPixelColor(i - 34 + 162, houseColor);
    strip.setPixelColor(i - 34 + 170, houseColor);
  }
  strip.show();
}
 */


/*
void tickVehicle1()
{
  static uint32_t previousMillis = 0;
  static byte currentPosition = 1;
  static byte direction = west;  // 0 north - 1 east - 2 south 3 west
  uint16_t wait = 500;
  int16_t newPosition = 4711;
  if (millis() - previousMillis > wait)
  {
    // aktuelle Position
    if (direction == north)
    {
      if (pgm_read_byte(&street[currentPosition].n) != 0)
        newPosition = pgm_read_byte(&street[currentPosition].n);
      else if (pgm_read_byte(&street[currentPosition].e) != 0)
      {
        Serial.println(F("go east"));
        newPosition = pgm_read_byte(&street[currentPosition].e);
        direction = east;
      }
      else if (pgm_read_byte(&street[currentPosition].w) != 0)
      {
        Serial.println(F("go west"));
        newPosition = pgm_read_byte(&street[currentPosition].w);
        direction = west;
      }
    }
    else if (direction == east)
    {
      if (pgm_read_byte(&street[currentPosition].e) != 0)
        newPosition = pgm_read_byte(&street[currentPosition].e);
      else if (pgm_read_byte(&street[currentPosition].s) != 0)
      {
        Serial.println(F("go south"));
        newPosition = pgm_read_byte(&street[currentPosition].s);
        direction = south;
      }
      else if (pgm_read_byte(&street[currentPosition].n) != 0)
      {
        Serial.println(F("go north"));
        newPosition = pgm_read_byte(&street[currentPosition].n);
        direction = north;
      }
    }
    else if (direction == south)
    {
      if (pgm_read_byte(&street[currentPosition].s) != 0)
        newPosition = pgm_read_byte(&street[currentPosition].s);
      else if (pgm_read_byte(&street[currentPosition].w) != 0)
      {
        Serial.println(F("go west"));
        newPosition = pgm_read_byte(&street[currentPosition].w);
        direction = west;
      }
      else if (pgm_read_byte(&street[currentPosition].e) != 0)
      {
        Serial.println(F("go east"));
        newPosition = pgm_read_byte(&street[currentPosition].e);
        direction = east;
      }
    }
    else if (direction == west)
    {
      if (pgm_read_byte(&street[currentPosition].w) != 0)
        newPosition = pgm_read_byte(&street[currentPosition].w);
      else if (pgm_read_byte(&street[currentPosition].n) != 0)
      {
        Serial.println(F("go north"));
        newPosition = pgm_read_byte(&street[currentPosition].n);
        direction = north;
      }
      else if (pgm_read_byte(&street[currentPosition].s) != 0)
      {
        Serial.println(F("go south"));
        newPosition = pgm_read_byte(&street[currentPosition].s);
        direction = south;
      }
    }
    Serial.print(F("vehicle1: from ")); Serial.print(currentPosition);
    Serial.print(F(" to ")); Serial.println(newPosition);
    currentPosition = newPosition;
    previousMillis = millis();

  }
}
*/

/*
 class TrafficLight
{
    uint32_t previousMillis = 0;
    byte currentPosition;
    byte state = 0;  // 0 red - 1 east - 2 south 3 west
    uint16_t wait = 5000;
    const uint32_t trafficRed = strip.Color(255,  0,   0);
    const uint32_t trafficGreen = strip.Color(0,  0,   0);
    const uint32_t trafficYellow = strip.Color(255,  100,   0);
  public:
    TrafficLight(uint32_t currentPosition):
      currentPosition(currentPosition)
    {}

    void setState(byte newState) {
      state = newState;
      switch (state)
      {
        case 0: strip.setPixelColor(currentPosition, trafficRed); break;
        case 2: strip.setPixelColor(currentPosition, trafficGreen); break;
        default: strip.setPixelColor(currentPosition, trafficYellow); // 1 u 2
      }
      strip.show();
    }

    void setInterval(uint16_t newInterval) {
      wait = newInterval;
    }

    void tick_auto()   // automatic running traficlight (if no traffic manager is used)
    {
      if (millis() - previousMillis > wait)
      {
        state++;
        state = state % 4;
        setState(state);
        previousMillis = millis();
      }
    }
};
*/

/*  
  TrafficLight trafficLight5(5);
  TrafficLight trafficLight21(21);
  TrafficLight trafficLight38(38);
  TrafficLight trafficLight39(39);

  TrafficLight trafficLight80(80);
  //TrafficLight trafficLight86(86);
  //TrafficLight trafficLight87(87);
  TrafficLight trafficLight94(94);
  //TrafficLight trafficLight101(101);
  TrafficLight trafficLight109(109);
  TrafficLight trafficLight114(114);
  //TrafficLight trafficLight122(122);
  TrafficLight trafficLight130(130);
  //TrafficLight trafficLight138(138);
  //TrafficLight trafficLight149(149);
  TrafficLight trafficLight157(157);
  TrafficLight trafficLight174(174);
  //TrafficLight trafficLight166(166);
  //TrafficLight trafficLight167(167);
  TrafficLight trafficLight160(160);
  TrafficLight trafficLight214(214);
  TrafficLight trafficLight215(215);
  TrafficLight trafficLight229(229);
  TrafficLight trafficLight245(245);
*/






/*
// die Traffic Manager als die Kreuzungen noch keine Objekte waren.
  void manageGroup1()
  {
  static uint32_t previousMillis = -10000;
  static byte state = 1;
  const uint16_t intervalShort = 1500;
  const uint16_t intervalLong = 4500;
  static uint16_t interval = intervalShort;
  if (millis() - previousMillis > interval)
  {
    state++;
    state = state % 4;
    switch (state)
    {
      case 0:
        trafficLight138.setState(0);
        trafficLight149.setState(0);
        trafficLight101.setState(0);
        trafficLight122.setState(0);
        trafficLight86.setState(2);
        trafficLight87.setState(2);
        trafficLight167.setState(2);
        trafficLight166.setState(2);
        interval = intervalLong;
        break;
      case 1:
      case 3:
        trafficLight138.setState(1);
        trafficLight149.setState(1);
        trafficLight101.setState(1);
        trafficLight122.setState(1);
        trafficLight86.setState(1);
        trafficLight87.setState(1);
        trafficLight167.setState(1);
        trafficLight166.setState(1);
        interval = intervalShort;
        break;
      default:
        trafficLight138.setState(2);
        trafficLight149.setState(2);
        trafficLight101.setState(2);
        trafficLight122.setState(2);
        trafficLight86.setState(0);
        trafficLight87.setState(0);
        trafficLight167.setState(0);
        trafficLight166.setState(0);
        interval = intervalLong;
    }
    previousMillis = millis();
  }
  }


void manageGroup2()
{
  static uint32_t previousMillis = -10000;
  static byte state = 0;
  if (millis() - previousMillis > 6000)
  {
    state = !state;
    trafficLight80.setState(state * 2);
    trafficLight174.setState(state * 2);
    trafficLight109.setState(!state * 2);
    trafficLight114.setState(!state * 2);
    previousMillis = millis();
  }
}

void manageGroup3()
{
  static uint32_t previousMillis = -10000;
  static byte state = 0;
  if (millis() - previousMillis > 5500)
  {
    state = !state;
    trafficLight214.setState(state * 2);
    trafficLight215.setState(state * 2);
    trafficLight245.setState(!state * 2);
    trafficLight229.setState(!state * 2);
    previousMillis = millis();
  }
}

void manageGroup4()
{
  static uint32_t previousMillis = -10000;
  static byte state = 0;
  if (millis() - previousMillis > 4500)
  {
    state = !state;
    trafficLight94.setState(state * 2);
    trafficLight160.setState(state * 2);
    trafficLight130.setState(!state * 2);
    trafficLight157.setState(!state * 2);
    previousMillis = millis();
  }
}

void manageGroup5()
{
  static uint32_t previousMillis = 0;
  static byte state = 0;
  if (millis() - previousMillis > 3500)
  {
    state = !state;
    trafficLight21.setState(state * 2);
    trafficLight5.setState(state * 2);
    trafficLight39.setState(!state * 2);
    trafficLight38.setState(!state * 2);
    previousMillis = millis();
  }
}


void arsonistTick()
{
  const byte action = HOUSE;                // The arsonist layes fires in Houses
  static uint32_t previousMillis = 0;
  static byte target = 0;
  static byte order = 0;
  static byte state = 0;                    // 0 ready // 1 Drive // 2 Arrival // 2 doing
  if (millis() - previousMillis > 500)      // yes, this action is slow - the car will block the street during the working time!
  {
    // suche ein Ziel
    if (state == 0)
    {
      byte i = random(0, noOfHouses);
      if (city.state[i] == action && random(0, 7) == 1)   // etwas abschwächen
      {
        target = pgm_read_byte(&houses[i][1]); // die Straße
        order = i;                             // die Haus-ID
        Serial.print(F("gehe zu @")); Serial.println(order);
        carArsonist.setTarget(target, order);
        state = 1;
      }
      else
      {
        carArsonist.setTarget(random(144, 159), 0);
      }
    }
    // Fahre mit dem Auto dort hin
    else if (state == 1)
    {
      // MISSING checken auf Zeitablauf ...
      if (carArsonist.currentPosition == target)
      {
        // Serial.println(F("arrived @")); Serial.println(pgm_read_byte(&houses[order][0]));
        state = 2;
      }
    }
    else if (state == 2)
    {
      // check if the house has still the right state and was not changed during our journey
      if (city.state[order] == action)
      {
        if (city.addDisaster (order))
        {
          Serial.print(F("started fire @")); Serial.println(pgm_read_byte(&houses[order][0]));
        }
        else
        {
          Serial.print(F("to late @")); Serial.println(pgm_read_byte(&houses[order][0]));
        }
      }
      state = 0;
    }
    previousMillis = millis();
  }
}

// returns the stack size ... not really usefull, will be 2048 on a UNO
static inline size_t stack_size()
{
  return RAMEND - SP;
}
*/
